"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getHighlighter = void 0;
const vscode_textmate_1 = require("vscode-textmate");
const languages_1 = require("./languages");
const resolver_1 = require("./resolver");
const onigLibs_1 = require("./onigLibs");
const themedTokenizer_1 = require("./themedTokenizer");
const renderer_1 = require("./renderer");
const themes_1 = require("./themes");
async function getHighlighter(options) {
    let t;
    if (typeof options.theme === 'string') {
        t = themes_1.getTheme(options.theme);
    }
    else if (options.theme.name) {
        t = options.theme;
    }
    else {
        t = themes_1.getTheme('nord');
    }
    let langs = [...languages_1.commonLangIds, ...languages_1.commonLangAliases];
    if (options.langs) {
        langs = options.langs;
    }
    const langRegistrations = languages_1.getLangRegistrations(langs);
    const s = new Shiki(t, langRegistrations);
    return await s.getHighlighter();
}
exports.getHighlighter = getHighlighter;
class Shiki {
    constructor(theme, langs) {
        this._resolver = new resolver_1.Resolver(langs, onigLibs_1.getOnigasm(), 'onigasm');
        this._registry = new vscode_textmate_1.Registry(this._resolver);
        this._registry.setTheme(theme);
        this._theme = theme;
        this._colorMap = this._registry.getColorMap();
        this._langs = langs;
    }
    async getHighlighter() {
        const ltog = {};
        await Promise.all(this._langs.map(async (l) => {
            const g = await this._registry.loadGrammar(l.scopeName);
            ltog[l.id] = g;
            l.aliases.forEach(la => {
                ltog[la] = g;
            });
        }));
        return {
            codeToThemedTokens: (code, lang) => {
                if (isPlaintext(lang)) {
                    throw Error('Cannot tokenize plaintext');
                }
                if (!ltog[lang]) {
                    throw Error(`No language registration for ${lang}`);
                }
                return themedTokenizer_1.tokenizeWithTheme(this._theme, this._colorMap, code, ltog[lang]);
            },
            codeToHtml: (code, lang) => {
                if (isPlaintext(lang)) {
                    return renderer_1.renderToHtml([[{ content: code }]], {
                        bg: this._theme.bg
                    });
                }
                if (!ltog[lang]) {
                    throw Error(`No language registration for ${lang}`);
                }
                const tokens = themedTokenizer_1.tokenizeWithTheme(this._theme, this._colorMap, code, ltog[lang]);
                return renderer_1.renderToHtml(tokens, {
                    bg: this._theme.bg
                });
            }
        };
    }
}
function isPlaintext(lang) {
    return ['plaintext', 'txt', 'text'].indexOf(lang) !== -1;
}
